package nl.amis.android;

import java.io.IOException;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import org.ksoap2.SoapEnvelope;
import org.ksoap2.serialization.PropertyInfo;
import org.ksoap2.serialization.SoapObject;
import org.ksoap2.serialization.SoapSerializationEnvelope;
import org.ksoap2.transport.HttpTransportSE;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapActivity;
import com.google.android.maps.MapController;
import com.google.android.maps.MapView;
import com.google.android.maps.Overlay;
import com.google.android.maps.OverlayItem;

import android.content.Context;
import android.graphics.drawable.Drawable;
import android.location.Address;
import android.location.Geocoder;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.Toast;

public class SpeedTraps extends MapActivity {

	public static Context mContext;
	private Drawable mDrawable;
	private MapController mMapController;
	private List<Overlay> mOverlays;
	private MyLocation mMyLocation;
	private final int GROUP_DEFAULT = 0;
	private final int MENU_RETRIEVE = 1, MENU_REPORT = 2;
	private static final String URL = "http://10.252.253.19:7101/SpeedTrapDataServices/SpeedTrapService?WSDL";
	// private static final String URL = "http://192.168.2.2:7101/SpeedTrapDataServices/SpeedTrapService?WSDL";

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);

		mContext = this;
		mDrawable = this.getResources().getDrawable(R.drawable.icon_oracle);

		MapView mMapView = (MapView) findViewById(R.id.map1);
		mMapView.setBuiltInZoomControls(true);
		mMapController = mMapView.getController();
		mMapController.setZoom(6);
		mOverlays = mMapView.getOverlays();
		// Current location
		mMyLocation = new MyLocation();
	}

	public boolean onCreateOptionsMenu(Menu menu) {
		menu.add(GROUP_DEFAULT, MENU_RETRIEVE, 0, R.string.omRetrieve).setIcon(
				R.drawable.ic_menu_refresh);
		menu.add(GROUP_DEFAULT, MENU_REPORT, 0, R.string.omReport).setIcon(
				R.drawable.ic_menu_mylocation);
		return super.onCreateOptionsMenu(menu);
	}

	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case MENU_RETRIEVE:
			retrieveSpeedTraps();
			return true;
		case MENU_REPORT:
			reportSpeedTrap();
			return true;
		}
		return super.onOptionsItemSelected(item);
	}

	private void retrieveSpeedTraps() {
		// Retrieves all the locations from the database with a soap call and
		// draws them as a marker on the map
		String SOAP_ACTION = "/nl/amis/speedtraps/common/findSpeedTrapsView";
		String NAMESPACE = "/nl/amis/speedtraps/common/types/";
		String METHOD_NAME = "findSpeedTrapsView";
		HttpTransportSE mHttpTransportSE = new HttpTransportSE(URL);
		SimpleDateFormat mSimpleDateFormat1 = new SimpleDateFormat(
				"yyyy-MM-dd'T'HH:mm:ss");
		SimpleDateFormat mSimpleDateFormat2 = new SimpleDateFormat(
				"EEE d MMM yyyy HH:mm:ss");
		try {
			// Compose the request
			SoapObject mRequest = new SoapObject(NAMESPACE, METHOD_NAME);
			// Put the request in an envelope
			SoapSerializationEnvelope mEnvelope = new SoapSerializationEnvelope(
					SoapEnvelope.VER11);
			mEnvelope.setOutputSoapObject(mRequest);
			// Make the soap call
			mHttpTransportSE.call(SOAP_ACTION, mEnvelope);
			// Remove all existing markers from the map
			mOverlays.clear();
			// Get the SoapResult from the envelope body
			SoapObject mResult = (SoapObject) mEnvelope.bodyIn;
			int count = (int) mResult.getPropertyCount();
			for (int i = 0; i < count; i++) {
				SoapObject mResultLocation = (SoapObject) mResult
						.getProperty(i);
				Double mResultLocationLat = new Double(mResultLocation
						.getProperty("Latitude").toString());
				Double mResultLocationLon = new Double(mResultLocation
						.getProperty("Longitude").toString());
				GeoPoint mResultLocationGeoPoint = new GeoPoint(
						(int) (mResultLocationLat * 1E6),
						(int) (mResultLocationLon * 1E6));
				// Draws a marker on the map for each location
				MarkerLayer mMarkerLayer = new MarkerLayer(mDrawable);
				String mStringCreationDate = mResultLocation.getProperty(
						"CreationDate").toString();
				Date mDateCreationDate = mSimpleDateFormat1
						.parse(mStringCreationDate.substring(0, 19));
				OverlayItem mOverlayItem = new OverlayItem(
						mResultLocationGeoPoint, mSimpleDateFormat2
								.format(mDateCreationDate),
						getAddressFromLocation(mResultLocationLat,
								mResultLocationLon));
				mMarkerLayer.addOverlayItem(mOverlayItem);
				mOverlays.add(mMarkerLayer);
				mMapController.animateTo(mResultLocationGeoPoint);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private void reportSpeedTrap() {
		// Stores the current location in the database with a soap call
		String SOAP_ACTION = "/nl/amis/speedtraps/common/createSpeedTrapsView";
		String NAMESPACE_1 = "/nl/amis/speedtraps/common/types/";
		String NAMESPACE_2 = "/nl/amis/speedtraps/common/";
		String METHOD_NAME_1 = "createSpeedTrapsView";
		String METHOD_NAME_2 = "speedTrapsView1";
		HttpTransportSE mHttpTransportSE = new HttpTransportSE(URL);
		try {
			// Compose the request with the current location
			SoapObject row = new SoapObject(NAMESPACE_1, METHOD_NAME_2);
			setProperty(row, NAMESPACE_2, "Latitude", ""
					+ roundDouble(mMyLocation.getmLocation().getLatitude(), 8));
			setProperty(row, NAMESPACE_2, "Longitude", ""
					+ roundDouble(mMyLocation.getmLocation().getLongitude(), 8));
			SoapObject mRequest = new SoapObject(NAMESPACE_1, METHOD_NAME_1);
			setProperty(mRequest, NAMESPACE_1, METHOD_NAME_2, row);
			// Put the request in an envelope
			SoapSerializationEnvelope mEnvelope = new SoapSerializationEnvelope(
					SoapEnvelope.VER11);
			mEnvelope.setOutputSoapObject(mRequest);
			// Make the soap call
			mHttpTransportSE.call(SOAP_ACTION, mEnvelope);
			Toast.makeText(mContext, "This speedtrap has been reported successfully.",
					Toast.LENGTH_SHORT).show();
		} catch (Exception e) {
			Toast.makeText(mContext, "This speedtrap has NOT been reported.",
					Toast.LENGTH_SHORT).show();
			e.printStackTrace();
		}
	}

	private String getAddressFromLocation(Double latitude, Double longitude) {
		// Translates a location to an address (reverse geocoding)
		StringBuilder mAddressSB = new StringBuilder("Address:\n");
		try {
			Geocoder mGeocoder = new Geocoder(mContext);
			List<Address> mAddresses = mGeocoder.getFromLocation(latitude,
					longitude, 1);
			if (mAddresses != null) {
				Address mAddress = mAddresses.get(0);
				for (int j = 0; j < mAddress.getMaxAddressLineIndex(); j++) {
					mAddressSB.append(mAddress.getAddressLine(j)).append("\n");
				}
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		return mAddressSB.toString();
	}

	private void setProperty(SoapObject soapObject, String namespace,
			String name, Object value) {
		PropertyInfo propertyInfo = new PropertyInfo();
		propertyInfo.setNamespace(namespace);
		propertyInfo.setName(name);
		if (value != null) {
			propertyInfo.setValue(value);
		}
		soapObject.addProperty(propertyInfo);
	}

	private static double roundDouble(double number, int decimalPlaces) {
		BigDecimal bd = new BigDecimal("" + number);
		bd = bd.setScale(decimalPlaces, BigDecimal.ROUND_HALF_UP);
		return bd.doubleValue();
	}

	@Override
	protected boolean isRouteDisplayed() {
		return false;
	}
}